Package: CERFIT
Version: 0.2.0
Title: Causal Effect Random Forest of Interaction Trees
Authors@R: c(
    person(given = "Justin",family = "Thorp",email = "jjtthorp@gmail.com",role = c("aut","cre")),
    person(given = "Joshua", family = "Moffat", email = "joshua.moffat@gmail.com", role = "aut"),
    person(given = "Luo",family = "Li",role = "aut"),
    person(given = "Juanjuan", family = "Fan",role = "aut"))
Description: Fits a Causal Effect Random Forest of Interaction Trees (CERFIT) which is a modification of the Random Forest algorithm where each split is chosen to maximize subgroup treatment heterogeneity. Doing this allows it to estimate the individualized treatment effect for each observation in either randomized controlled trial (RCT) or observational data. For more information see L. Li, R. A. Levine, and J. Fan (2022) <doi:10.1002/sta4.457>.
License: GPL (>= 2)
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
LinkingTo: Rcpp, RcppArmadillo
Imports: partykit, CBPS, randomForest, twang, Rcpp, stats, glmnet,
        survival
Depends: R (>= 2.10)
NeedsCompilation: yes
Packaged: 2026-05-07 14:44:17 UTC; justinthorp
Author: Justin Thorp [aut, cre],
  Joshua Moffat [aut],
  Luo Li [aut],
  Juanjuan Fan [aut]
Maintainer: Justin Thorp <jjtthorp@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-07 16:43:33 UTC
Built: R 4.4.3; x86_64-apple-darwin20; 2026-05-07 18:53:26 UTC; unix
Archs: CERFIT.so.dSYM
