# Generated by roxygen2: do not edit by hand

S3method(predict,CERFIT)
S3method(print,CERFIT)
export(CERFIT)
export(MinDepth)
importFrom(Rcpp,sourceCpp)
importFrom(grid,depth)
importFrom(partykit,as.constparty)
importFrom(partykit,fitted_node)
importFrom(partykit,info_node)
importFrom(partykit,kidids_split)
importFrom(partykit,nodeapply)
importFrom(partykit,nodeids)
importFrom(partykit,party)
importFrom(partykit,partynode)
importFrom(partykit,partysplit)
importFrom(partykit,split_node)
importFrom(stats,complete.cases)
importFrom(stats,terms)
useDynLib(CERFIT, .registration = TRUE)
