# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

find_split <- function(y, x, trt, cutpts, method, propensity, minbucket, response_type) {
    .Call(`_CERFIT_find_split`, y, x, trt, cutpts, method, propensity, minbucket, response_type)
}

